/**
 * Name: Particle.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __PARTICLE_H__
#define __PARTICLE_H__

#ifdef __APPLE__
#include <GraphicsLib/Vector.h>
#include <GraphicsLib/Colour.h>
#include <GraphicsLib/Point3.h>
#include <GraphicsLib/Util.h>
#else
#include "Vector.h"
#include "Colour.h"
#include "Point3.h"
#include "Util.h"
#endif

#include "Solver.h"


namespace ParticleSystems {
	
	//! The Particle Class
	/**
	* The Particle Class is the Superclass for all Particles. It provides 
	* Constructors as well as abstract methods that need to be implemented by
	* all Subclasses.
	* @author Hannes Ricklefs
	*/
	class Particle {
	
	public:
		
		//! The minimal Constructor
		/**
		* This Constructor only takes the ParticleSystem ID and the 
		* Solver to update the Particles position.
		* \param sysId the ParticleSystem ID this Particle belongs to
		* \param sol a pointer to the Solver
		*/
		Particle(int sysId, Solver *sol);
		
		//! The main Constructor
		/**
		* This Constructor is the main Constructor for a Particle. It sets the 
		* most important attributes of a Particle. 
		* \param sysId the ParticleSystem ID this Particle belongs to
		* \param sol a pointer to the Solver
		* \param pos the initial Position
		* \param vel the inital Velocity
		* \param c the initial Colour
		* \param fc the fade Colour
		* \param size the inital Size
		* \param fsize the fade Size
		* \param maxLife the maximum Lifespan of the Particle
		*/
		Particle(   int sysId,
					Solver *sol,
					GraphicsLib::Point3 pos,
                    GraphicsLib::Vector vel,
                    GraphicsLib::Colour c,
					GraphicsLib::Colour fc,
					GLfloat size,
					GLfloat fsize,
					GLfloat maxLife
				);
		
		//! The Collision Constructor
		/**
		* A Constructor to set attributes that are needed for Collision
		* \param sysId the ParticleSystem ID this Particle belongs to
		* \param sol a pointer to the Solver
		* \param pos the initial Position
		* \param vel the inital Velocity
		* \param c the initial Colour
		* \param trans the transparency
		* \param b the bounceness
		* \param ma the mass
		* \param mo the momentum
		* \param e the energy
		* \param s the size
		*/
        Particle(   int sysId,
					GraphicsLib::Point3 pos,
                    GraphicsLib::Vector vel,
                    GraphicsLib::Colour c,
                    GLfloat trans,
                    float b,
                    float ma,
                    float mo,
                    float e,
                    float s
                );

		//! The Destructor
		/**
		* Destructs the Particle
		*/
		virtual ~Particle();
		
		//! An abstract Function
		/**
		* Draws the Particle
		*/
		virtual void draw() = 0;
		
		//! An abstract Function
		/**
		* Updates a Particles 
		*/
		virtual void update() = 0;
		
		//! An abstract Function
		/**
		* Returns true if the Particle is dead
		* \return the value if the Particle is dead
		*/
		virtual bool isDead() = 0;
		
		//! A public Variable
		/**
		* The current life of the Particle
		*/
		GLfloat life;
		
		//! A public Variable
		/**
		* The Maximum Lifespan of the Particle
		*/
		GLfloat MAXLIFE;
		
		//! A public Variable
		/**
		* The ParticleSystem ID this Particle belongs to
		*/
		int SYSID;
		
		//! A public Variable
		/**
		* The Solver for this Particle to update the Particles position
		*/
        Solver *solv;
		
		//! A public Variable
		/**
		* The current position of the Particle
		*/
		GraphicsLib::Point3 position;
		
		//! A public Variable
		/**
		* The old_Position of the Particle
		*/
		GraphicsLib::Point3 old_Position;
		
		//! A public Variable
		/**
		* The speed of the Particle
		*/
		GraphicsLib::Vector velocity;
		
		//! A public Variable
		/**
		* The acceleration of the Particle
		*/
		GraphicsLib::Vector accel;
		
		//! A public Variable
		/**
		* The colour of the Particle
		*/
		GraphicsLib::Colour colour;
		
		//! A public Variable
		/**
		* The fade colour of the Particle
		* If set to zero nothing happens
		*/
		GraphicsLib::Colour faded_colour;
		
		//! A public Variable
		/**
		* The transparency of the Particle
		*/
		GLfloat transparency;
		
		//! A public Variable
		/**
		* The bouncyness of the Particle when colliding
		*/
		float bounce;
		
		//! A public Variable
		/**
		* The Mass of the Particle
		*/
		float mass;
		
		//! A public Variable
		/**
		* The momentum of the Particle
		*/
		float momentum;
		
		//! A public Variable
		/**
		* The energy of the Particle
		*/
		float energy;
		
		//! A public Variable
		/** 
		* The size of the Particle
		*/
		GLfloat size;
		
		//! A public Variable
		/**
		* The amount the Particle gets reduced each update. 
		* If set to zero nothing happens.
		*/
		GLfloat flare_size;
		
		//! A public Variable
		/**
		* Specifies if the Particle is dead
		* True if Particle is dead
		*/
		bool dead;
		
	};
}
#endif

		
