/**
 * Name: ParticleSystem.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __PARTICLESYSTEM_H__
#define __PARTICLESYSTEM_H__

#ifdef __APPLE__
	#include <GraphicsLib/Point3.h>
	#include <GraphicsLib/Material.h>
#else
	#include "Material.h"
	#include "Point3.h"
#endif

#include <vector>
#include "Debugger.h"

#include "Particle.h"
#include "Solver.h"
#include "Emitter.h"
#include "DataStructure.h"
#include "Force.h"
#include "Texture.h"




namespace ParticleSystems {
	//! A typedef
	/**
	* To store the ParticleSystem ID
	*/
	typedef int SYSTEMID;
	
	//! The ParticleSystem Class
	/**
	* The ParticleSystem enables ParticleSystems to be created. 
	* It incooperates all the elements that enable ParticleSystems
	* to emitt, store and evolve/update Particles.
	* @author Hannes Ricklefs
	*/
    class ParticleSystem {
	
	public:
		//! The Constructor
		/**
		* Creates a new ParticleSystem by passing an Ident, Emitter, DataStructure, Solver, the inital
		* number of Particles, and the number of Particles to be created per Frame.
		* \param ident  the ParticleSystems ID
		* \param emit a Pointer to an Emitter to emit new Particles
		* \param dataStruc a Pointer to a DataStructure to store all the Particles
		* \param s a Pointer to a Solver to evolve the Particles
		* \param num the number of inital Particles
		* \param var the variance of new Particles per Frame
		*/
		ParticleSystem(int ident, Emitter *emit, DataStructure *dataStruc, Solver *s, int num, int var);
		
		//! The Destructor
		/**
		* Destructs the ParticleSystem
		*/
		~ParticleSystem();
		
		//! A member Function
		/**
		* Stops the ParticleSystem and kills all Particles
		*/
		void die();
		
		//! A member Function
		/**
		* Restarts the ParticleSystem
		*/
		void restart();
		
		//! A member Function
		/**
		* Pauses the ParticleSystem
		*/
		void pause();
		
		//! A member Function
		/**
		* Evolves the ParticleSystem
		*/
		void evolve();
		
		//! A member Function
		/**
		* Creates new Particles according to the 
		* passed number
		* \param number the number of new Particles
		*/
		void createNewParticles(int number);
		
		//! A member Function
		/**
		* Creates one new Particle and adds it to the DataStructure
		* It checks if a new Particle needs to be created or if a dead
		* Particle can be reborne
		*/
		void newParticle(void);
		
		//! A member Function
		/**
		* Kills all Particles in the DataStructure
		*/
		void killParticles(void);
		
		//! A member Function
		/**
		* Adds a Solver to the ParticleSystem
		* \param s a Pointer to the Solver to be added
		*/
		void addSolver(Solver *s);
        
		//! A member Function
		/**
		* Draws the ParticleSystem
		*/
		void draw();
		
		//! A member Function
		/**
		* Returns if the ParticleSystem is still alive
		* \return true if the ParticleSystem is still alive
		*/
		bool stillAlive(void);
		
		//! A public Variable
		/**
		* The texture for all ImgParticles
		*/
		GraphicsLib::Material texture;
		
		//! A public Variable
		/**
		* The identifier of this ParticleSystem
		*/
		SYSTEMID identify;
		
		//! A public Variable
		/**
		* Stores if the ParticleSystem is still alive
		*/
		bool alive;
		
		//! A public Variable
		/**
		*The number of initial particles
		*/
		int numInitParticles;
		
		//! A public Variable
		/**
		* The maximum number of new Particles per evolution.
		*/
		int varNewParticles;
		
		//! A public Variable
		/**
		* The position of the particle system
		*/
		GraphicsLib::Point3 position;

		//! A public Variable
		/**
		* The Emitter for all Particles
		*/
		Emitter *em;
		
		//! A private Variable
		/**
		* The DataStructure to store all the Particles
		*/
		DataStructure *data;
		
		//! A public Variable
		/**
		* All the Solvers for this ParticleSystem
		*/
		std::vector<Solver *> sList;
		
		//! A public Variable
		/**
		* The iterator to go through all the Solvers
		*/
		std::vector<Solver *>::iterator sListItt;
		
		//! A public Variable
		/**
		* Stores if the ParticleSystem is currently paused
		*/
		bool paused;
	};
}
#endif

