/**
 * Name: VectorDS.h
 *
 * Copyright (c) 2005 University of Bournemouth
 */
#ifndef __VECTORDS_H__
#define __VECTORDS_H__

#include <vector>
#include <iostream>

#ifdef __APPLE__
#include <GraphicsLib/Point3.h>
#include <GraphicsLib/ThreadTimer.h>
#else
#include "ThreadTimer.h";
#include "Point3.h"
#endif

#include "DataStructure.h"
#include "Particle.h"
#include "CollisionObject.h"

#include "Debugger.h"


namespace ParticleSystems {
	//! A typedef for all the Particles
	typedef std::vector<Particle *> allP;
	
	//! A typedef for an itterator for all the Particles
	typedef std::vector<Particle *>::iterator pAllItt;
	
	//! A typedef for all the dead Particles
	typedef std::vector<Particle *> dead;
	
	//! A typedef for an iterator for all the dead Particles
	typedef std::vector<Particle *>::iterator allDeadItt;

	//! A typedef for all the CollisionObjects
	typedef std::vector<CollisionObject *> cObjects;
	
	//! A typedef for an itterator for all the CollisionObjects
	typedef std::vector<CollisionObject *>::iterator allcObjects;
	
	//! The VectorDS Class
	/**
	* The VectorDS Class extends the DataStructure Class. The VectorDS Class
	* uses the vector container Class to store all the Particles and CollisionObjects.
	* @see DataStructure
	* @author Hannes Ricklefs
	*/
	class VectorDS : public DataStructure {
		
		public:
			
			//! The Constructor
			/**
			* Constructs a new VectorDS
			*/
			VectorDS();
			
			//! The Destructor
			/**
			* Destructs a VectorDS
			*/
			virtual ~VectorDS();
			
			//! A virtual Function
			/**
			* Adds a new Particle.
			* \param particle a pointer to the new Particle
			*/
			virtual void addParticles(Particle *particle);
			
			//! A virtual Function
			/**
			* Removes all Particles
			*/
			virtual void killParticles();
			
			//! A virtual Function
			/**
			* Evolves the Particles within the VectorDS
			*/
			virtual void evolve();
			
			//! A virtual Function
			/**
			* Checks is the given point is colliding with any of the 
			* CollisionObjects.
			* \param pos the position to be checked
			* \return True if in collision range
			*/
			virtual bool positionCheck(GraphicsLib::Point3 pos);
			
			//! A virtual Function
			/**
			* Draws all the Particle and CollisionObjects within this
			* VectorDS
			*/
            virtual void draw();
			
			//! A virtual Function
			/** 
			* Return the number of Particles within this VectorDS
			* as an int.
			* \return The number of Particles
			*/
			virtual int numParticles();
			
			//! A virtual Function
			/**
			* Returns true if the VectorDS contains any
			* dead Particles
			* \return True is dead Particles are stored
			*/
			virtual bool hasDead();
			
			//! A virtual Function
			/**
			* Returns a pointer to a dead Particle
			* !! Should only be used if hasDead returns True !!
			* Otherwise NULL will be returned.
			* \return A pointer to a dead Particle.
			*/
			virtual Particle* getDead();
			
			//! A virtual Function
			/**
			* Add a new CollisionObject to the VectorDS
			* \param co a pointer to the CollisionObject
			*/
            virtual void addCollisionObject(CollisionObject *co);
			
			//! A virtual Function
			/**
			* Deletes the given CollisionObject
			* \param co a pointer to the about to be deleted CollisionObject
			*/
            virtual void deleteCollisionObject(CollisionObject *co);
			
			//! The typdef definition
			/**
			* Stores all the Particles
			*/
			allP particles;	
			
			//! The typdef definition
			/**
			* Stores all the dead Particles
			*/
			dead deadParticles;
            
			//! The typdef definition
			/**
			* Stores all the CollisionObjects
			*/
			cObjects collisionObjects;

	};
}
#endif


